package com.example.nto.domain.entity;


import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.hibernate.annotations.GenericGenerator;

import javax.persistence.*;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.util.List;

@Data
@Entity
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Table(name = "terminals")
public class Terminal {
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private long id;

    @Column(name = "name")
    @NotBlank(message = "Название не может быть пустым!")
    @Size(max = 100, message = "Максимальная длина названия 100 символов!")
    private String name;

    // Мне нужна была рандомная генерация кода для терминала, так что я мог сделать кривую реализацию через самописную функцию,
    // но боялся, что она будет генерировать не уникальные значения. (я очень смутно представляю, как эта штука работает)
    // Код взят отсюда: https://stackoverflow.com/questions/25082244/auto-generate-unique-random-string-in-spring-mvc-hibernate
    @GenericGenerator(name = "uuid-gen", strategy = "uuid")
    @GeneratedValue(generator = "uuid-gen")
    @NotBlank(message = "Код не может быть пустым!")
    @Column(name = "code", nullable = false, unique = true)
    private String code;

    // todo: Протестировать нужен ли тут каскад тип
    @ManyToOne(fetch = FetchType.LAZY, cascade = CascadeType.REMOVE)
    @JoinColumn(name = "office_id", nullable = false)
    private Office office;

    @OneToMany(mappedBy = "startTerminal")
    private List<Visit> startVisits;

    @OneToMany(mappedBy = "endTerminal")
    private List<Visit> endVisits;
}
